package geoinformatics1;

import java.util.*;


import repast.simphony.context.*;
import repast.simphony.context.space.grid.*;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.grid.*;
import static geoinformatics1.CellState.*;
import static java.lang.Math.*;

public class CityInitialiser extends DefaultContext<Cell> implements ContextBuilder<Cell>
{
	Grid<Cell> grid;
	Context<Cell> context;
	List<int[]> CENTRAL_POINT = new ArrayList<int[]>();
	List<int[]> LINE = new ArrayList<int[]>();
	List<int[]> CIRCLE = new ArrayList<int[]>();
	List<int[]> STAR = new ArrayList<int[]>();
	int height;
	int width;
	

	public Context<Cell> build( Context<Cell> context )
	{
		this.context = context;
		init();
		
		/************ This is the only thing you should change in here ***************/
		//Change this line to point to one of the other shapes to alter the initial seed
		for( int[] coords : CENTRAL_POINT ) setState( NEW, coords );
		
		return context;
	}
	
	/**
	 * Basic initialisation
	 */
	void init()
	{
		//get the size of the grid from the environment
		Parameters p = RunEnvironment.getInstance().getParameters();
		height = (Integer) p.getValue( "worldHeight" );
		width = (Integer) p.getValue( "worldWidth" );

		// Create a new 2D grid 
		GridFactory factory =  GridFactoryFinder.createGridFactory( null );
		grid = factory.createGrid( "Grid", context, new GridBuilderParameters<Cell>( new StrictBorders(), new SimpleGridAdder<Cell>(), false, width, height ) );

		for( int i = 0; i < width; i++ )
			for( int j = 0; j < height; j++ )
				setState( EMPTY, i, j );
		
		initialiseShapes();
	}
	
	/**
	 * Sets the state of one of the cells in the grid
	 * @param state
	 * @param pos
	 */
	void setState(  CellState state, int... pos )
	{
		Cell c = grid.getObjectAt( pos );
		if( c == null )
		{
			c = new Cell();
			context.add( c );
			grid.getAdder().add( grid, c );
			grid.moveTo( c, pos );
		}
		c.setState( state );
		c.updateState();
		
	}
	
	/**
	 * Initialises the possible starting shapes
	 */
	void initialiseShapes()
	{
		int minDim = min( width, height );
		int cx = width/2;
		int cy = height/2;
		
		//Central point is just one coordinate
		CENTRAL_POINT.add( new int[]{ cx, cy } );
		//Horizontal line from 1/4 to 3/4 of the width
		for( int i = width/4; i < 3*width/4; i++ )
			LINE.add( new int[]{i, cy } );
		
		//All points within 1/6 of the centre
		for( int i = 0; i < width; i++ )
			for( int j = 0; j < height; j++ )
				if( ( cx - i ) * (cx-i) + (cy-j)* (cy-j) < minDim / 6 )
					CIRCLE.add( new int[] { i, j } );
		
		//Points on diagonals
		STAR.add( new int[]{ cx, cy } );
		for( int i = 1; i < minDim/6; i++ )
		{
			STAR.add( new int[]{ cx-i, cy-i } );
			STAR.add( new int[]{ cx+i, cy-i } );
			STAR.add( new int[]{ cx+i, cy+i } );
			STAR.add( new int[]{ cx-i, cy+i } );
		}
		
	}
}
