package geoinformatics3.agents;

import java.util.*;

import geoinformatics3.CombinedModel;
import geoinformatics3.city.*;
import repast.simphony.context.Context;
import repast.simphony.util.ContextUtils;

public class AgentComplete extends Agent
{
	public static int MATURATION_AGE = 20;
	public static int OLD_AGE = 50;
	public static int MAX_CELLS = 50;
	int age = 1;
	Cell location = null;
	Context context;
	Demographics demographics;
	CombinedModel model;
	City city;
	Comparator<Cell> comparator;
	
	

	public AgentComplete( CombinedModel model )
	{
		//Hack, to make the complete class separate!
		super( model );
		this.model = model;
		this.context = model.getContext();
		this.demographics = model.getDemographics();
		this.city = model.getCity();
	}
	
	
	/*
	 * Useful functions here
	 */
	
	public void die()
	{
		Context c = getContext();
		//System.out.println( "Agent " + this + " dying in context " + c.hashCode() + " with location " + location  );
		remove();
	}
	
	public void moveOrEmigrate()
	{
		Cell c = findNewLocation();
		if( c != null && moveTo( c ) ) return;
		//Try to move, otherwise, emigrate
		System.out.println( "No free cells! Emigrating");
		model.getCity().unsatisfiedDemmand();
		emigrate();
	}
	
	public void tryMoving()
	{
		//Try to find a new place to live
		Cell c = findNewLocation();
		if( c == null ) return;
		if( evaluateCell( c ) > evaluateCell( location ) )
			moveTo( c );
	}
	
	
	/*
	 * You probably don't need functions under here...
	 */

	public void setAge( int age )
	{
		this.age = age;
	}
	
	public int getAge()
	{
		return this.age;
	}
	
	Context getContext()
	{
		Context c = (Context)ContextUtils.getContext( this );
		return c;
	}
	

	void remove()
	{
		if( location != null ) location.remove( this );
		//else System.out.println( "Couldn't remove agent from location! (" + location +")" );
		location = null;
		
		Context<AgentComplete> c = getContext();
		if( c != null ) c.remove( this );
		else System.out.println( "Couldn't remove agent from context!");
		if( c != null && c.contains( this )) System.out.println( "Still in context after removal!");
		
	}
	
	public int getType() {return 0;}
	
	public void emigrate()
	{
		System.out.println( "Agent emigrating");
		remove();
	}
	
	boolean moveTo( Cell c )
	{
		if( c.add( this ) )
		{
			if( location != null ) location.remove( this );
			location = c;
			return true;
		}
		return false;
	}

	public Cell getLocation()
	{
		return location;
	}
	
	Cell findNewLocation()
	{
		List<Cell> cells = new ArrayList( MAX_CELLS );
		for( Cell c : city.getRandomAvailableCells( MAX_CELLS ) )
			cells.add( c );
		Collections.sort( cells, getComparator() );
		for( Cell c : cells )
		{
			if( c.hasFreeCapacity() ) return c;
		}
		return null;
	}

	public double evaluateCell( Cell c ) { return 0.6; }
	
	public Comparator<Cell> getComparator()
	{
		if( comparator == null )
		comparator = new Comparator<Cell>()
		{
			public int compare( Cell o1, Cell o2 )
			{
				return Double.compare( evaluateCell( o1 ), evaluateCell( o2 ) );
			}
			
		};
		
		return comparator;
	}
}
