package geoinformatics3.agents;

import geoinformatics3.CombinedModel;
import repast.simphony.context.*;
import repast.simphony.context.space.grid.*;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.*;
import repast.simphony.engine.schedule.*;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.grid.*;
import repast.simphony.util.ContextUtils;
import static java.lang.Math.*;

public class Demographics 
{
	Context context;
	public static int IMMIGRATION = 100;
	public static int INITIAL_MATURES = 1000;
	int badAgents = 0;
	int population = 0;
	int matures = 0;
	int olds = 0;
	int young = 0;
	CombinedModel model;
	
	public Demographics( CombinedModel model )
	{
		this.model = model;
	}
	
	public void initParams()
	{
		try{
			Parameters p = RunEnvironment.getInstance().getParameters();
			} catch( Exception e )
			{
				System.err.println( "Could not find demographics params in the parameters");
			}
	}
	
	public void initParams( int immigration,  int initialPop )
	{
		IMMIGRATION = immigration;
		INITIAL_MATURES = initialPop;
	}

	public void build( Context context )
	{
		System.err.println( "Setting context" + this.hashCode() + "-" + context.hashCode() );
		this.context = context;
	
		
		
		for( int i = 0; i < INITIAL_MATURES; i++ )
			addMature();
		context.add( this );
		//RunState.getInstance().getScheduleRegistry().getModelSchedule().schedule( this );
	}
	
	@ScheduledMethod(start=1,interval=1, priority = 3 )
	public void migration()
	{
		System.out.println( "Migration happening");
		for( int i = 0; i < IMMIGRATION; i++ ) addMature();
	}
	
	@ScheduledMethod(start=1,interval=1,priority=ScheduleParameters.LAST_PRIORITY )
	public void updateAggregate()
	{
		population = context.getObjects( Agent.class ).size();
		matures = context.getObjects( Mature.class ).size();
		olds = context.getObjects( Old.class ).size();
		young = context.getObjects( Young.class ).size();
		System.out.printf( "Population: %d, (y: %d, m: %d, o: %d)\n", population, young, matures, olds );
	}
	@ScheduledMethod(start=1,interval=1,priority=ScheduleParameters.LAST_PRIORITY )
	public void checkLocations()
	{
		System.out.println( "Checking context " + context.hashCode() );
		badAgents = 0;
		for( Object o : context.getAgentLayer( Agent.class ) )
		{
			Agent a = (Agent)o;
			if( a.getLocation() == null )
			{
				badAgents++;
				System.out.println( "Agent " + a + " has no location!");
			}
		}
		if( badAgents > 0 ) System.err.println( "Got " + badAgents + " agents with no location!" );
	}
	
	@ScheduledMethod( start=1, interval=1,priority=ScheduleParameters.FIRST_PRIORITY )
	public void spacer()
	{
		System.out.println( "++++++++++++++++++++++++++++++++++++++\n\n\n" );
		System.out.println( "**************************************\n\n\n" );
	}
	
	public int getBadAgents()
	{
		return badAgents;
	}
	
	public void forceAgeless()
	{
		int i = 0;
		for( Object o : context.getAgentLayer( Agent.class ) )
		{
			i++;
			Agent a = (Agent)o;
			if( a instanceof Mature ) a.setAge( 30 );
			if( a instanceof Old ) a.setAge( 60 );
			if( a instanceof Young ) a.setAge( 10 );
		}
		System.out.println( "Set age for " + i + " agents");
	}
	
	public void addMature()
	{
		Agent m = new Mature( model );
		m.setAge( RandomHelper.nextIntFromTo( Agent.MATURATION_AGE, Agent.OLD_AGE-1 ) );
		addAgent( m );
	}
	
	public void addAgent( Agent a )
	{
		System.out.println( "Adding agent");
		context.add( a );
		a.moveOrEmigrate();
	}

	public int getMatures()
	{
		return matures;
	}

	public int getOlds()
	{
		return olds;
	}

	public int getYoung()
	{
		return young;
	}
	
	
}
