package geoinformatics3.agents;


import geoinformatics3.CombinedModel;
import geoinformatics3.city.Cell;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;

public class MatureComplete extends Agent
{
	public static double MORTALITY_RATE = 0.005;
	public static double BIRTH_RATE = 0.045;
	public static double SATISFACTION_THRESHOLD = 0.8;
	
	public MatureComplete( CombinedModel model )
	{
		super( model );
	}
	
	@ScheduledMethod( start=1,interval=1)
	public void update()
	{
		age++;
		if( age > OLD_AGE ) becomeOld();
		if( RandomHelper.nextDouble() < MORTALITY_RATE ) die();
		if( RandomHelper.nextDouble() < BIRTH_RATE ) giveBirth();
		if( location != null && evaluateCell( location ) < SATISFACTION_THRESHOLD ) tryMoving();
	}
	
	public Old becomeOld()
	{
		Old old = new Old( model );
		old.setAge( age );
		getContext().add( old );
		Cell loc = location;
		remove();
		if( loc != null )
		{
			old.moveOrEmigrate();
			//if( ! old.moveTo( loc ) )
				//System.err.println( "Could not move to current location when becoming old!");
		}
		return old;
	}
	
	public Young giveBirth()
	{
		Young y = new Young( model );
		Context c = getContext();
		if( c != null ) c.add( y );
		y.location = location;
		return y;
	}
	
	public int getType() {return 1;}
	
	public static double doEvaluation( Cell c )
	{
		return c.getNeighbourhoodDensity() / Cell.CURRENT_MAXIMUM_CAPACITY;
	}
	
}
