package geoinformatics3.agents;

import geoinformatics3.CombinedModel;
import geoinformatics3.city.Cell;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;

public class OldComplete extends Agent
{
	public static double MORTALITY_RATE = 0.01;
	public static int MAX_AGE = 120;
	public static double SATISFACTION_THRESHOLD = 0.8;
	
	public OldComplete( CombinedModel model )
	{
		super( model );
	}
	
	/**
	 * Init block - reads parameters from the model.score
	 */
	{
		try{
		Parameters p = RunEnvironment.getInstance().getParameters();
		MORTALITY_RATE = (Double) p.getValue( "oldMortality" );
		} catch( Exception e )
		{
			System.err.println( "Could not find mature params in the parameters");
		}
	}
	
	@ScheduledMethod( start=1,interval=1)
	public void update()
	{
		age++;
		if( RandomHelper.nextDouble() < MORTALITY_RATE ) die();
		if( age > MAX_AGE ) die();
		if( location != null && evaluateCell( location ) < SATISFACTION_THRESHOLD ) tryMoving();
	}

	public int getType() {return 2;}
	
	public double evaluateCell( Cell c )
	{
		return doEvaluation( c );
	}
	
	public static double doEvaluation( Cell c )
	{
		if( ! c.isHabitable() ) return 0;
		return 1-c.getNeighbourhoodDensity()/Cell.CURRENT_MAXIMUM_CAPACITY;
	}
}
