package geoinformatics3.city;

import java.awt.Color;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;

import repast.simphony.visualization.visualization2D.style.Style2D;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;
import geoinformatics3.agents.Agent;
import static geoinformatics3.city.CellState.*;

/**  
 * Colours squares according to their state: grey for empty, yellow for available,
 * red for new and established cells fading from blue to green as they approach
 * redevelopment age. 
 *
 * @author Dave Murray-Rust
 */

public abstract class AbstractCellStyle implements Style2D<Cell> {

	Rectangle2D rect = new Rectangle2D.Float(0, 0, 10, 10);

	Shape s = new Rectangle2D.Float(0,0,10,10);

	Stroke stroke = null;
	Paint strokePaint = null;

	double rotation = 0.0;

	public AbstractCellStyle() {
		super();
	}

	// Sets how the node is initially drawn (for new agents)
	public PNode getPNode(Cell agent, PNode node) {
		PPath path = new PPath(s);
		path.setBounds(rect);
		path.setPaint(this.getPaint(agent));
		path.setStroke(this.getStroke(agent));
		path.setStrokePaint(this.getStrokePaint(agent));

		return path;
	}

	public Paint getPaint(Cell agent) {
		return getColor( agent );
	}
	
	abstract Color getColor( Cell agent );

	public double getRotation(Cell agent) {
		return rotation;
	}
	public void setRotation(double rot) {
		this.rotation = rot;
	}
	public void setBounds(Rectangle2D bounds) {
		this.rect = bounds;
	}
	public Rectangle2D getBounds(Cell agent) {
		return rect;
	}
	public Stroke getStroke(Cell agent) {
		return stroke;
	}
	public boolean isScaled(Cell agent) {
		return true;
	}
	public Paint getStrokePaint(Cell agent) {
		return strokePaint;
	}

	public PText getLabel(Cell object) {
		// TODO Auto-generated method stub
		return null;
	}
}
