package geoinformatics3;

import geoinformatics3.agents.Mature;
import geoinformatics3.city.Cell;

import org.junit.Test;
import static org.junit.Assert.*;

public class CapcityTests extends ModelTest
{
	Cell c1;
	Cell c2;
	Cell c3;
	Cell c4;
	
	@Test
	public void testCapcityRatio()
	{
		createChangelessWorld( 2, true );
		Cell.INITIAL_CAPACITY = 1;
		Cell.CAPACITY_INCREMENT = 1;
		Cell.MAX_CAPACITY = 10;
		city.reinitialiseCellCapacities();
		c1 = grid.getObjectAt( 0,0 );
		c2 = grid.getObjectAt( 0,1 );
		c3 = grid.getObjectAt( 1,0 );
		c4 = grid.getObjectAt( 1,1 );
		
		doAssertion( 0, 0, 0, 0, 0, 0, 0, 0 );
		
		Mature m = new Mature( model );
		assertTrue( c1.add( m ) );
		assertEquals( 1, c1.getPopulation() );
		assertEquals( 1, c1.getCapacity() );
		doAssertion( 1, 0, 0, 0, 0.25, 0.25, 0.25, 0.25 );
		stepChangeless( 10 );
		assertEquals( 3, city.getTotalFreeCapacity() );
		assertEquals( 4, city.getTotalCapacity() );
		doAssertion( 1, 0, 0, 0, 0.25, 0.25, 0.25, 0.25 );
		
		
		m = new Mature( model );
		assertFalse( c1.add( m ) );
		assertTrue( c2.add( m ) );
		doAssertion( 1, 1, 0, 0, 0.5, 0.5, 0.5, 0.5 );
		stepChangeless( 10 );
		doAssertion( 1, 1, 0, 0, 0.5, 0.5, 0.5, 0.5 );
		
		c1.incrementCapacity();
		c2.incrementCapacity();
		c3.incrementCapacity();
		c4.incrementCapacity();
		doAssertion( 0.5, 0.5, 0, 0, 0.25, 0.25, 0.25, 0.25 );
		
		stepChangeless( 10 );
		
		m = new Mature( model );
		assertTrue( c1.add( m ) );
		assertEquals( 2, c1.getPopulation() );
		assertEquals( 2, c1.getCapacity() );
		doAssertion( 1.0, 0.5, 0, 0, 0.375, 0.375, 0.375, 0.375 );
		m = new Mature( model );
		assertFalse( c1.add( m ) );
		assertTrue( c2.add( m ) );
		stepChangeless( 10 );
		doAssertion( 1, 1, 0, 0, 0.5, 0.5, 0.5, 0.5 );
		
		assertTrue( c3.add( new Mature( model )));
		assertTrue( c3.add( new Mature( model )));
		assertFalse( c3.add( new Mature( model )));
		assertTrue( c4.add( new Mature( model )));
		assertTrue( c4.add( new Mature( model )));
		assertFalse( c4.add( new Mature( model )));
		doAssertion( 1, 1, 1,1,1,1,1,1 );
	}
	
	@Test
	public void testCapacityOnLargeBoard()
	{
		createChangelessWorld( 7, true );
		Cell.INITIAL_CAPACITY = 1;
		Cell.CAPACITY_INCREMENT = 1;
		Cell.MAX_CAPACITY = 10;
		city.reinitialiseCellCapacities();
		c1 = grid.getObjectAt( 0,3 );
		c2 = grid.getObjectAt( 1,3 );
		c3 = grid.getObjectAt( 2,3 );
		c4 = grid.getObjectAt( 3,3 );
		
		assertTrue( c1.add(  new Mature( model ) ));
		assertEquals( 1, c1.getPopulation() );
		assertEquals( 1, c1.getCapacity() );
		doAssertion( 1, 0, 0,0,1/9.0,1/9.0,0,0 );
		assertFalse( c1.add(  new Mature( model ) ));
		assertTrue( c2.add(  new Mature( model ) ));
		assertEquals( 1, c2.getPopulation() );
		assertEquals( 1, c2.getCapacity() );
		doAssertion( 1, 1, 0,0,2/9.0,2/9.0,1/9.0,0 );
		
	}

	void doAssertion( double c1c, double c2c, double c3c, double c4c, double c1n, double c2n, double c3n, double c4n )
	{
		assertEquals( c1c, c1.getCapacityRatio() );
		assertEquals( c2c, c2.getCapacityRatio() );
		assertEquals( c3c, c3.getCapacityRatio() );
		assertEquals( c4c, c4.getCapacityRatio() );
		assertEquals( c1n, c1.getNeighbourhoodCapacityRatio() );
		assertEquals( c2n, c2.getNeighbourhoodCapacityRatio() );
		assertEquals( c3n, c3.getNeighbourhoodCapacityRatio() );
		assertEquals( c4n, c4.getNeighbourhoodCapacityRatio() );
		
	}
}
