package geoinformatics3;

import geoinformatics3.agents.Demographics;
import geoinformatics3.city.*;

import java.io.File;

import repast.simphony.context.Context;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.grid.Grid;

import org.junit.*;

public class ModelTest
{
	public static RUnitRunner runner = new RUnitRunner();
	public static Context mainContext;
	static boolean loaded = false;
	String modelDir = "geoinformatics3.rs";
	protected static Parameters params = null;
	Demographics demographics;
	City city;
	CombinedModel model;
	Grid<Cell> grid = null;
	
	public ModelTest( )
	{
		//this.modelDir = modelDir;
		//initModel( modelDir );
	}
	
	void loadModel()
	{
		loadModel( modelDir );
	}
	
	void loadModel( String modelDir )
	{
		try
		{
			System.out.print( "Loading Model");
			runner.load( new File( modelDir ) );
			System.out.println( "...OK");
			loaded = true;
			params = runner.getParameters();
		} catch (Exception e)
		{
			System.err.println( "Could not create model..." + e );
			e.printStackTrace();
		}
	}
	
	public void initModel()
	{
		runner.runInitialize();
		mainContext = runner.getContext();
		model = CombinedModel.instance;
		demographics = model.getDemographics();
		city = model.getCity();
		System.out.println( "City is now: " + city );
		grid = (Grid<Cell>)mainContext.getProjection( City.CITY_GRID );
	}
	
	protected void createChangelessWorld( int size )
	{
		createChangelessWorld( size, false );
	}
	
	protected void createChangelessWorld( int size, boolean allDeveloped )
	{
		loadModel();
		params.setValue( "worldSize", size );
		params.setValue( "immigration", 0 );
		params.setValue( "emigration", 0 );
		params.setValue( "initialPopulation", 0 );
		params.setValue( "youngMortality", 0 );
		params.setValue( "matureMortality", 0 );
		params.setValue( "oldMortality", 0 );
		params.setValue( "birthRate", 0 );
		initModel();
		if( allDeveloped ) for( Cell c : city.getCells() )
		{
			c.setState( CellState.NEW );
			c.updateState();
		}
	}
	
	protected void step()
	{
		step( 1 );
	}
	
	protected void step( int numSteps )
	{
		for( int i = 0; i < numSteps; i++ ) runner.step();
	}
	
	protected void stepChangeless()
	{
		stepChangeless( 1 );
	}
	protected void stepChangeless( int numSteps )
	{
		for( int i = 0; i < numSteps; i++ )
		{
			runner.step();
			demographics.forceAgeless();
		}
	}

}
