package geoinformatics4;

import java.io.IOException;

import geoinformatics4.agents.*;
import geoinformatics4.city.*;
import repast.simphony.context.*;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.*;
import repast.simphony.engine.schedule.*;
import repast.simphony.parameter.Parameters;

public class CombinedModel extends DefaultContext implements ContextBuilder
{
	Demographics demographics;
	City city;
	Context context;
	public static CombinedModel instance = null;
	boolean updateParams = true;

	public Context build( Context context )
	{
		this.context = context;
		instance = this;
		setupParameters();
		Schedule sche = new Schedule();
		System.out.println( "Schedule actions - model: " + sche.getModelActionCount() + ", non model: " + sche.getActionCount() + sche.getClass());
		RunState.getInstance().getScheduleRegistry().setModelSchedule( sche );
		System.out.println( "Creating city... ");
		context.clear();
		city = new City( this );
		city.build( context );
		System.out.println( "City created OK (" + city.hashCode() + ")");
		
		System.out.println( "Creating initial population... ");
		demographics = new Demographics( this );
		demographics.build( context );
		System.out.println( "Population created OK");
		
		
		sche.schedule( this );
		return context;
	}
	
	@ScheduledMethod( start = 1, interval=1, priority=ScheduleParameters.FIRST_PRIORITY )
	public void modelUpdate()
	{
		if( updateParams ) setupParameters();
	}
	
	private void setupParameters()
	{
		try{
			Parameters p = RunEnvironment.getInstance().getParameters();
			Cell.MAX_CAPACITY = (Integer) p.getValue( "maxCapacity" );
			Cell.INITIAL_CAPACITY = (Integer) p.getValue( "initialCapacity" );
			Cell.CAPACITY_INCREMENT = (Integer) p.getValue( "capacityIncrement" );
			Cell.SURPLUS_FACTOR = (Double) p.getValue( "surplusHousingFactor" );
			Cell.RELEASE_PROBABILITY = (Double) p.getValue( "releaseProbability" );
			Cell.REDEVELOPMENT_RATE = (Double) p.getValue( "redevelopmentRate" );
			Cell.REDEVELOPMENT_AGE = (Integer) p.getValue( "redevelopmentAge" );
			
			
			Mature.MORTALITY_RATE = (Double) p.getValue( "matureMortality" );
			Mature.BIRTH_RATE = (Double) p.getValue( "birthRate" );
			Young.MORTALITY_RATE = (Double) p.getValue( "youngMortality" );
			Old.MORTALITY_RATE = (Double) p.getValue( "oldMortality" );
			Demographics.IMMIGRATION = (Integer) p.getValue( "immigration" );
			Demographics.INITIAL_MATURES = (Integer) p.getValue( "initialPopulation" );
			Demographics.NUM_FRIENDS = (Integer) p.getValue( "numberOfFriends" );
			Agent.SATISFACTION_THRESHOLD = (Double) p.getValue( "satisfactionThreshold");
			Agent.SOCIAL_PREFERENCE_WEIGHTING = (Double) p.getValue( "socialPreferenceWeighting");
			Agent.DENSITY_PREFERENCE_WEIGHTING = (Double) p.getValue( "densityPreferenceWeighting");
			Agent.TYPE_PREFERENCE_WEIGHTING = (Double) p.getValue( "typePreferenceWeighting");
		} catch( Exception e )
		{
			System.err.println( "Missing parameter!");
			e.printStackTrace();
		}
	}

	public Demographics getDemographics()
	{
		return demographics;
	}

	public City getCity()
	{
		return city;
	}

	public Context getContext()
	{
		return context;
	}

	public boolean isUpdateParams()
	{
		return updateParams;
	}

	public void setUpdateParams( boolean updateParams )
	{
		this.updateParams = updateParams;
	}

}
