package geoinformatics4.agents;

import java.util.*;

import geoinformatics4.CombinedModel;
import geoinformatics4.city.*;
import repast.simphony.context.Context;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.space.graph.*;
import repast.simphony.space.grid.Grid;
import repast.simphony.util.ContextUtils;

public class AgentComplete extends Agent
{
	public static final String SOCIAL_NETWORK = "Social Network";
	public static int MATURATION_AGE = 20;
	public static int OLD_AGE = 50;
	public static int MAX_CELLS = 20;
	int age = 1;
	Cell location = null;
	Context context;
	Demographics demographics;
	CombinedModel model;
	City city;
	Comparator<Cell> comparator;
	AgentType type;
	public static double SATISFACTION_THRESHOLD = 0.0;
	public static double TYPE_PREFERENCE_WEIGHTING = 1.1;
	public static double DENSITY_PREFERENCE_WEIGHTING = 1.0;
	public static double SOCIAL_PREFERENCE_WEIGHTING = 1.0;
	Network<Agent> socialNetwork;

	public AgentComplete( CombinedModel model )
	{
		this( model, AgentType.getRandomType() );
	}

	public AgentComplete( CombinedModel model, AgentType type )
	{
		//Hack!
		super( model, type );
		this.model = model;
		this.context = model.getContext();
		this.demographics = model.getDemographics();
		this.city = model.getCity();
		this.type = type;
		socialNetwork = demographics.getNetwork();
	}

	/**
	 * Add code in here to check every turn whether the current cell is
	 * satisfactory, using the satisfaciton threshold
	 */
	@ScheduledMethod(start = 1, interval = 1)
	public void checkEvaluation()
	{
		if( location != null && evaluateCell( location ) < SATISFACTION_THRESHOLD )
			tryMoving();
	}

	/**
	 * You should create your evaluation function here, using some weighted combination of
	 * the type- density- and social preference functions later in this file
	 * @param c
	 * @return
	 */
	public double evaluateCell( Cell c )
	{
		return TYPE_PREFERENCE_WEIGHTING * getTypePreference( c ) +
			DENSITY_PREFERENCE_WEIGHTING * getDensityPreference( c ) +
			SOCIAL_PREFERENCE_WEIGHTING * getSocialPreference( c );
	}

	/**
	 * YOu should put your social preference function in here. It may be slightly complicated, so a few
	 * useful objects are provided. You should handle cases like agents in the network not having a location
	 * (may happen as they are leaving town/dieing) and not having any friends
	 * HINT: socialNetwork.getAdjacent( this ) will get all the agents this one is linked to
	 * HINT: grid.getDistance() will return the distance between two GridPoints. getLocation() in Cell will provide
	 * the GridPoint a cell is in.
	 * @param c
	 * @return
	 */
	public double getSocialPreference( Cell c )
	{
		if( c == null ) return 0;
		socialNetwork = demographics.getNetwork();
		Grid<Cell> grid = city.getGrid();
		double ssd = 0;
		int num = 0;
		for ( Agent e : socialNetwork.getAdjacent( this ) )
		{
			if( e.getLocation() == null )
				continue;
			ssd += grid.getDistance( c.getLocation(), e.getLocation().getLocation() );
			num++;
		}
		if( num == 0 )
			return 0;
		return 1 - ssd / ( num * city.getSize() );
	}

	/**
	 * This is a wrapper round the static method - you should use this function
	 * in your evaluation function, but you should put the actual preference function in
	 * the static version below.
	 * @param c
	 * @return
	 */
	public double getTypePreference( Cell c )
	{
		return typeBasedPreference( c, type );
	}

	/**
	 * Create a function here which indicates a preference for neighbourhoods of the
	 * given type.
	 * @param c
	 * @param type
	 * @return
	 */
	public static double typeBasedPreference( Cell c, AgentType type )
	{
		return c.getProportionOfTypeInNeighbourhood( type );
	}

	/**
	 * This is already overridden in Mature and Old, so you don't need to change it!
	 * @param c
	 * @return
	 */
	public double getDensityPreference( Cell c )
	{
		return 0.5;
	}


	/*
	 * Useful functions here
	 */

	public void die()
	{
		Context c = getContext();
		remove();
	}
	public void moveOrEmigrate()
	{
		Cell c = findNewLocation();
		//if( c != null && ( location == null || evaluateCell( c ) > evaluateCell( location ) ) && moveTo( c ) )
		if( c != null && moveTo( c ) )
			return;
		//Try to move, otherwise, emigrate
		System.out.println( "No free better cells! Emigrating" );
		model.getCity().unsatisfiedDemmand();
		emigrate();
	}

	public void tryMoving()
	{
		//Try to find a new place to live
		Cell c = findNewLocation();
		if( c == null )
			return;
		if( evaluateCell( c ) > evaluateCell( location ) )
			moveTo( c );
	}

	/*
	 * You probably don't need functions under here...
	 */

	public void setAge( int age )
	{
		this.age = age;
	}

	public int getAge()
	{
		return this.age;
	}

	Context getContext()
	{
		Context c = (Context) ContextUtils.getContext( this );
		return c;
	}

	void remove()
	{
		if( location != null )
			location.remove( this );
		//else System.out.println( "Couldn't remove agent from location! (" + location +")" );
		location = null;

		Context<AgentComplete> c = getContext();
		if( c != null )
			c.remove( this );
		else
			System.out.println( "Couldn't remove agent from context!" );
		if( c != null && c.contains( this ) )
			System.out.println( "Still in context after removal!" );

	}

	public void emigrate()
	{
		System.out.println( "Agent emigrating" );
		remove();
	}

	public boolean moveTo( Cell c )
	{
		if( c.add( this ) )
		{
			if( location != null )
				location.remove( this );
			location = c;
			return true;
		}
		return false;
	}

	public Cell getLocation()
	{
		return location;
	}

	Cell findNewLocation()
	{
		List<Cell> cells = new ArrayList( MAX_CELLS );
		for ( Cell c : city.getRandomAvailableCells( MAX_CELLS ) )
			cells.add( c );
		Collections.sort( cells, getComparator() );
		Collections.reverse( cells );
		for ( Cell c : cells )
		{
			if( c.hasFreeCapacity() )
				return c;
		}
		return null;
	}

	public Comparator<Cell> getComparator()
	{
		if( comparator == null )
			comparator = new Comparator<Cell>()
			{
				public int compare( Cell o1, Cell o2 )
				{
					return Double.compare( evaluateCell( o1 ), evaluateCell( o2 ) );
				}

			};

		return comparator;
	}

	public void setType( AgentType type )
	{
		this.type = type;
	}

	public AgentType getType()
	{
		return type;
	}

	void replaceInNetwork( AgentComplete previous )
	{
		socialNetwork = demographics.getNetwork();
		for ( Agent e : socialNetwork.getAdjacent( previous ) )
			socialNetwork.addEdge( this, e );
	}

}
