package geoinformatics4.agents;

import java.util.*;

import geoinformatics4.CombinedModel;
import repast.simphony.context.*;
import repast.simphony.context.space.graph.NetworkBuilder;
import repast.simphony.context.space.grid.*;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.*;
import repast.simphony.engine.schedule.*;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.*;
import repast.simphony.util.ContextUtils;
import repast.simphony.util.collections.*;
import repast.simphony.visualization.editor.AddListener;
import static java.lang.Math.*;

public class DemographicsComplete 
{
	Context context;
	public static int IMMIGRATION = 100;
	public static int INITIAL_MATURES = 1000;
	int badAgents = 0;
	int population = 0;
	int matures = 0;
	int olds = 0;
	int young = 0;
	CombinedModel model;
	Network<Agent> network;
	List<Agent> newborns = new ArrayList<Agent>();
	public static int NUM_FRIENDS = 2;
	
	public DemographicsComplete( CombinedModel model )
	{
		this.model = model;
	}
	
	/**
	 * Create friendships here.
	 * HINT: newborns contains a list of the newly born agents
	 * HINT: use getRandomSample to get an appropriate size sample of agents
	 * HINT: make sure you're not creating links between an agent and itself...
	 * HINT: use createLink to create links. 
	 * 
	 */
	@ScheduledMethod( start=1.05, interval=1 )
	public void createFriendships()
	{
		for( Agent a : newborns )
		{
			int numLinks  = 0;
			for( Agent b : getRandomSample( newborns, NUM_FRIENDS ))
			{
				if( createLink( a, b ) ) numLinks++;
			}
		}
	}
	
	/**
	 * Make sure that agents coming into the system have some friends!
	 * HINT: context.getRandomObjects( Agent.class, number ) will get you some
	 * random agents to become friendly with.
	 */
	public void addMature()
	{
		Agent m = new Mature( model );
		m.setAge( RandomHelper.nextIntFromTo( Agent.MATURATION_AGE, Agent.OLD_AGE-1 ) );
		
		for( Agent f : (Iterable<Agent>)context.getRandomObjects( Agent.class, 3 ))
			createLink( m, f );
		addAgent( m );
	}
	
	Iterable<Agent> getRandomSample( List<Agent> agents, int number )
	{
		return new RandomIterable<Agent>(new ListIndexedIterable<Agent>( newborns ), number );
	}
	
	
	boolean createLink( Agent a, Agent b )
	{
		if( ! context.contains( a ) || ! context.contains( b )) return false;
		if( network.isAdjacent( a, b ) ) return false;
		
		network.addEdge( a, b );
		return true;
	}
	
	@ScheduledMethod( start=1.5, interval=1 )
	public void clearNewborns()
	{
		newborns.clear();
	}
	
	public void initParams( int immigration,  int initialPop )
	{
		IMMIGRATION = immigration;
		INITIAL_MATURES = initialPop;
	}

	public void build( Context context )
	{
		System.err.println( "Setting context" + this.hashCode() + "-" + context.hashCode() );
		this.context = context;
	
		
		
		for( int i = 0; i < INITIAL_MATURES; i++ )
			addMature();
		context.add( this );
		
		
		NetworkBuilder<Agent> builder = new NetworkBuilder<Agent>(Agent.SOCIAL_NETWORK, context, false );
		network = builder.buildNetwork();
		
		//RunState.getInstance().getScheduleRegistry().getModelSchedule().schedule( this );
	}
	
	@ScheduledMethod(start=1,interval=1, priority = 3 )
	public void migration()
	{
		System.out.println( "Migration happening");
		for( int i = 0; i < IMMIGRATION; i++ ) addMature();
	}
	
	@ScheduledMethod(start=1,interval=1,priority=ScheduleParameters.LAST_PRIORITY )
	public void updateAggregate()
	{
		population = context.getObjects( Agent.class ).size();
		matures = context.getObjects( Mature.class ).size();
		olds = context.getObjects( Old.class ).size();
		young = context.getObjects( Young.class ).size();
		System.out.printf( "Population: %d, (y: %d, m: %d, o: %d)\n", population, young, matures, olds );
	}
	@ScheduledMethod(start=1,interval=1,priority=ScheduleParameters.LAST_PRIORITY )
	public void checkLocations()
	{
		System.out.println( "Checking context " + context.hashCode() );
		badAgents = 0;
		for( Object o : context.getAgentLayer( Agent.class ) )
		{
			Agent a = (Agent)o;
			if( a.getLocation() == null )
			{
				badAgents++;
				System.out.println( "Agent " + a + " has no location!");
			}
		}
		if( badAgents > 0 ) System.err.println( "Got " + badAgents + " agents with no location!" );
	}
	
	@ScheduledMethod( start=1, interval=1,priority=ScheduleParameters.FIRST_PRIORITY )
	public void spacer()
	{
		System.out.println( "++++++++++++++++++++++++++++++++++++++\n\n\n" );
		System.out.println( "**************************************\n\n\n" );
	}
	
	
	public int getBadAgents()
	{
		return badAgents;
	}
	
	public void forceAgeless()
	{
		int i = 0;
		for( Object o : context.getAgentLayer( Agent.class ) )
		{
			i++;
			Agent a = (Agent)o;
			if( a instanceof Mature ) a.setAge( 30 );
			if( a instanceof Old ) a.setAge( 60 );
			if( a instanceof Young ) a.setAge( 10 );
		}
		System.out.println( "Set age for " + i + " agents");
	}
	
	
	public void addAgent( Agent a )
	{
		System.out.println( "Adding agent to context " + context.hashCode());
		context.add( a );
		a.moveOrEmigrate();
	}

	public int getMatures()
	{
		return matures;
	}

	public int getOlds()
	{
		return olds;
	}

	public int getYoung()
	{
		return young;
	}

	public Network<Agent> getNetwork()
	{
		return network;
	}
	
	
	public void addNewborn( Young y )
	{
		newborns.add( y );
	}
	
}
