package geoinformatics4;

import geoinformatics4.agents.*;
import geoinformatics4.city.*;

import org.junit.*;
import static org.junit.Assert.*;

import repast.simphony.dataLoader.ContextBuilder;

public class BasicTests extends ModelTest
{
	
	@Test
	public void testSingleCellAddition()
	{
		Cell.MAX_CAPACITY = 10;
		Cell.INITIAL_CAPACITY = 10;
		createChangelessWorld( 1 );
		Cell c = grid.getObjectAt( 0, 0 );
		
		for( int i = 0; i < Cell.INITIAL_CAPACITY; i++ )
		{
			demographics.addMature();
			stepChangeless();
			assertEquals( i+1, c.getPopulation() );
		}
		demographics.addMature();
		stepChangeless(); 
		assertEquals( Cell.INITIAL_CAPACITY, c.getPopulation() );
		demographics.addMature();
		stepChangeless(); 
		assertEquals( Cell.INITIAL_CAPACITY, c.getPopulation() );
		
	}

	@Test
	public void testSingleCellDeath()
	{
		createChangelessWorld( 1 );
		Cell c = grid.getObjectAt( 0, 0 );
		Agent a = new Mature( model );
		a.setAge( 30 );
		demographics.addAgent( a );
		stepChangeless();
		assertEquals( 1, c.getPopulation() );
		assertEquals( c, a.getLocation() );
		a.die();
		assertEquals( 0, c.getPopulation() );
		assertNull( a.getLocation() );
		stepChangeless();
	}
	
	@Test
	public void testSingleCellManyDeaths()
	{
		Cell.INITIAL_CAPACITY = 10;
		Cell.MAX_CAPACITY = 10;
		createChangelessWorld( 1 );
		Cell c = grid.getObjectAt( 0, 0 );
		for( int i = 0; i < 5; i++ )
		{
			Agent[] agents = new Agent[Cell.INITIAL_CAPACITY];
			for( int j = 0; j < Cell.INITIAL_CAPACITY; j++ )
				agents[j] = new Mature( model );
			for( Agent a : agents )
			{
				a.setAge( 30 );
				demographics.addAgent( a );
			}
			stepChangeless();
			assertEquals( Cell.INITIAL_CAPACITY, c.getPopulation() );
			for( Agent a : agents ) assertEquals( c, a.getLocation() );
			for( Agent a : agents ) a.die();
			stepChangeless();
			for( Agent a : agents ) assertNull( a.getLocation() );
			assertEquals( 0, c.getPopulation() );
			stepChangeless();
		}
	}
	
	@Test
	public void testNaturalDeath()
	{
		Cell.INITIAL_CAPACITY = 10;
		Cell.MAX_CAPACITY = 10;
		System.out.println( "********************\nNatural Deaths\n********************");
		createChangelessWorld( 2, true );
		assertEquals( 0, mainContext.getObjects( Agent.class ).size() );
		
		city.printState();
		
		System.out.println( "-----------------\nAdding agents\n" );
		for( int i = 0; i < 4 * Cell.INITIAL_CAPACITY; i++ )
		{
			Agent a = new Old( model );
			a.setAge( Old.MAX_AGE - 2 );
			demographics.addAgent( a );
		}
		for( int i = 0; i < 2; i++ )
			for( int j = 0; j < 2; j++ )
				assertEquals( Cell.INITIAL_CAPACITY, grid.getObjectAt( i,j ).getCapacity() );
		System.out.println( "-----------------\nRunning sim\n" );
		for( int i = 0; i < 1; i++ )
		{
			step();
			assertEquals( 0, demographics.getBadAgents() );
		}
	}
}
