package geoinformatics4;

import java.io.File;

import repast.simphony.batch.BatchScenarioLoader;
import repast.simphony.context.Context;
import repast.simphony.engine.controller.*;
import repast.simphony.engine.environment.*;
import repast.simphony.engine.schedule.*;
import repast.simphony.parameter.*;
import simphony.util.messages.MessageCenter;

public class RUnitRunner extends AbstractRunner
{
	private static MessageCenter msgCenter = MessageCenter.getMessageCenter( RUnitRunner.class );

	private RunEnvironmentBuilder runEnvironmentBuilder;
	protected Controller controller;
	protected boolean pause = false;
	protected Object monitor = new Object();
	protected SweeperProducer producer;
	private ISchedule schedule;
	private Parameters params;
	

	public RUnitRunner()
	{
		runEnvironmentBuilder = new DefaultRunEnvironmentBuilder( this, true );
		controller = new DefaultController( runEnvironmentBuilder );
		controller.setScheduleRunner( this );
	}

	public void load( File scenarioDir ) throws Exception
	{
		System.out.println( "Loading scenario");
		BatchScenarioLoader loader;
		if( scenarioDir.exists() )
		{
			loader = new BatchScenarioLoader( scenarioDir );
			ControllerRegistry registry = loader.load( runEnvironmentBuilder );
			controller.setControllerRegistry( registry );
			params = loader.getParameters();
		} else
		{
			msgCenter.error( "Scenario not found", new IllegalArgumentException( "Invalid scenario " + scenarioDir.getAbsolutePath() ) );
			return;
		}

		controller.batchInitialize();
		Parameters p =controller.runParameterSetters( loader.getParameters() );
		System.out.println( "Done loading scenario");
	}

	public void runInitialize()
	{
		controller.runInitialize( null );
		schedule = RunState.getInstance().getScheduleRegistry().getModelSchedule();
	}
	
	public Context getContext()
	{
		if( controller != null ) return controller.getCurrentRunState().getMasterContext();
		return null;
		
	}

	public void cleanUpRun()
	{
		controller.runCleanup();
	}

	public void cleanUpBatch()
	{
		controller.batchCleanup();
	}

	// returns the tick count of the next scheduled item
	public double getNextScheduledTime()
	{
		return ( (Schedule) RunEnvironment.getInstance().getCurrentSchedule() ).peekNextAction().getNextTime();
	}

	// returns the number of model actions on the schedule
	public int getModelActionCount()
	{
		return schedule.getModelActionCount();
	}

	// returns the number of non-model actions on the schedule
	public int getActionCount()
	{
		return schedule.getActionCount();
	}

	// Step the schedule
	public void step()
	{
		schedule.execute();
	}
	
	public void runUntil( double timestep )
	{
		while( getNextScheduledTime() < timestep )
			step();
	}

	// stop the schedule
	public void stop()
	{
		if( schedule != null )
			schedule.executeEndActions();
	}

	public void setFinishing( boolean fin )
	{
		schedule.setFinishing( fin );
	}

	public void execute( RunState toExecuteOn )
	{
		// required AbstractRunner stub.  We will control the
		//  schedule directly.
	}
	
	public Parameters getParameters() { return params; }

}
