package geoinformatics4;

import geoinformatics4.agents.*;
import geoinformatics4.city.Cell;

import org.junit.Test;
import static org.junit.Assert.*;

import repast.simphony.space.graph.*;

public class SocialNetworkTest extends ModelTest
{
	Network<Agent> net;
	
	@Test
	public void testNetworkTransitions()
	{
		createChangelessWorld( 1 );
		Cell.INITIAL_CAPACITY = 10;
		city.reinitialiseCellCapacities();
		
		System.out.println(mainContext.hashCode() );
		net = demographics.getNetwork();
		Mature m1 = new Mature(model);
		demographics.addAgent( m1 );
		assertTrue( mainContext.contains( m1 ));
		
		Mature m2 = new Mature(model);
		demographics.addAgent( m2 );
		assertTrue( mainContext.contains( m2 ));
		
		Mature m3 = new Mature(model);
		demographics.addAgent( m3 );
		assertTrue( mainContext.contains( m3 ));
		
		net.addEdge( m1, m2 );
		net.addEdge( m3, m1 );
		
		assertTrue( net.isAdjacent( m1, m2 ) );
		assertTrue( net.isAdjacent( m1, m3 ) );
		assertFalse( net.isAdjacent( m2, m3 ) );
		assertDirectionalLink( m1, m2 );
		assertDirectionalLink( m3, m1 );
		
		Old o1 = m1.becomeOld();
		assertTrue( net.isAdjacent( o1, m2 ) );
		assertTrue( net.isAdjacent( o1, m3 ) );
		assertFalse( net.isAdjacent( m2, m3 ) );
		assertDirectionalLink( o1, m2 );
		assertDirectionalLink( m3, o1 );
		
		Young y = m2.giveBirth();
		net.addEdge( y, m2 );
		net.addEdge( m2, y );
		assertDirectionalLink( y, m2 );
		assertDirectionalLink( m2, y );
		
		m1 = y.mature();
		assertDirectionalLink( m1, m2 );
		assertDirectionalLink( m2, m1 );
		
		o1 = m2.becomeOld();
		assertDirectionalLink( m1, o1 );
		assertDirectionalLink( o1, m1 );
		
	}
	
	@Test
	public void testSSDFunction()
	{
		createChangelessWorld( 2, true );
		net = demographics.getNetwork();
		Cell c1 = grid.getObjectAt( 0, 0 );
		Cell c2 = grid.getObjectAt( 1, 0 );
		Cell c3 = grid.getObjectAt( 0, 1 );
		Cell c4 = grid.getObjectAt( 1, 1 );
		/**
		 * c1  c2
		 * c3  c4
		 */
		assertTrue( c1.hasFreeCapacity() );
		assertEquals( 1, c1.getCapacity()  );
		
		Agent a1 = addMature();
		if( a1.getLocation() != c1 ) assertTrue( a1.moveTo( c1 ) );
		Agent a2 = addMature();
		if( a2.getLocation() != c2 ) assertTrue( a2.moveTo( c2 ) );
		Agent a3 = addMature();
		if( a3.getLocation() != c3 ) assertTrue( a3.moveTo( c3 ) );
		Agent a4 = addMature();
		if( a4.getLocation() != c4 ) assertTrue( a4.moveTo( c4 ) );
		
		assertEquals( 1, a1.getSocialPreference( c1 ) );
		assertEquals( 1, a1.getSocialPreference( c2 ) );
		assertEquals( 1, a1.getSocialPreference( c3 ) );
		assertEquals( 1, a1.getSocialPreference( c4 ) );
		
		//Now add a link to a2, in c2
		net.addEdge( a1, a2 );
		//should be 1 unit away in 1 and 4 -> 1/1+1 = 0.5
		assertEquals( 0.5, a1.getSocialPreference( c1 ) );
		assertEquals( 0.5, a1.getSocialPreference( c4 ) );
		//should be 0 unit away in 2 -> 1/1+0 = 1
		assertEquals( 1.0, a1.getSocialPreference( c2 ) );
		//should be 2 away in 3 -> 1 / 1+2 = 0.33
		assertEquals( 1.0/3, a1.getSocialPreference( c3 ) );
		
		//Adding another one in c3 should
		net.addEdge( a1, a3 );
		assertEquals( 0.5, a1.getSocialPreference( c1 ) );
		assertEquals( 0.5, a1.getSocialPreference( c2 ) );
		assertEquals( 0.5, a1.getSocialPreference( c3 ) );
		assertEquals( 0.5, a1.getSocialPreference( c4 ) );
		
		
	}
	
	void assertDirectionalLink( Agent source, Agent target )
	{
		for( RepastEdge<Agent> e : net.getOutEdges( source ) )
			if( e.getTarget() == target ) return;
		fail();
	}

}
