package geoinformatics4;

import geoinformatics4.agents.*;
import geoinformatics4.city.Cell;

import org.junit.Test;
import static org.junit.Assert.*;


public class TransitionTests extends ModelTest
{
	
	@Test
	public void testGettingOld()
	{
		Cell.INITIAL_CAPACITY = 1;
		createChangelessWorld( 1, true );
		Cell c = grid.getObjectAt( 0,0 );
		Mature m = new Mature( model );
		m.setAge( 30 );
		AgentType type = m.getType();
		demographics.addAgent( m );
		assertEquals( 1, c.getPopulation() );
		
		step();
		assertEquals( 1, c.getPopulation() );
		
		Old o = m.becomeOld();
		assertEquals( c, o.getLocation() );
		assertNull( m.getLocation() );
		assertEquals( m.getAge(), o.getAge() );
		assertEquals( 1, c.getPopulation() );
		assertEquals( type, m.getType() );
		
		step();
		o.die();
		assertNull( o.getLocation() );
		assertEquals( 0, c.getPopulation() );
	}
	
	@Test
	public void testGivingBirth()
	{
		createChangelessWorld( 1, true );
		
		Cell.INITIAL_CAPACITY = 2;
		Cell.MAX_CAPACITY = 2;
		city.reinitialiseCellCapacities();
		Cell c = grid.getObjectAt( 0,0 );
		Mature m = new Mature( model );
		AgentType type = m.getType();
		m.setAge( 30 );
		demographics.addAgent( m );
		assertEquals( 1, c.getPopulation() );
		
		step();
		assertEquals( 1, c.getPopulation() );
		
		Young y = m.giveBirth();
		step();
		assertEquals( 1, c.getPopulation() );
		assertEquals( 1, y.getAge() );
		assertEquals( c, y.getLocation() );
		assertEquals( type, y.getType() );
		
		Mature m2 = y.mature();
		step();
		assertEquals( 2, c.getPopulation() );
		assertEquals( 2, m2.getAge() );
		assertEquals( c, m2.getLocation() );
		
		m.die();
		step();
		assertEquals( 1, c.getPopulation() );
		
		m2.die();
		step();
		assertEquals( 0, c.getPopulation() );
	}
}
