package geoinformatics4;

import java.util.Map;

import org.junit.Test;
import static org.junit.Assert.*;

import geoinformatics4.agents.*;
import geoinformatics4.city.Cell;

public class TypesTests extends ModelTest
{
	Cell cell;
	AgentType[] types = AgentType.values();
	
	@Test
	public void testMajorities()
	{
		createBasic( 1 );
		assertNull( cell.getMajorityType() );
		
		addMature( types[0] );
		assertEquals( types[0], cell.getMajorityType() );
		addMature( types[1] );
		addMature( types[1] );
		assertEquals( types[1], cell.getMajorityType() );
	}
	
	@Test
	public void testNeighbourhoodProportions()
	{
		createBasic( 2 );
		Cell c2 = grid.getObjectAt( 1, 1 );
		AgentType a = types[0];
		AgentType b = types[1];
		stepChangeless();
		Mature mA = addMature( a );
		Map<AgentType, Integer> cTypes = cell.getNeighbourhoodTypes();
		assertEquals( 1, cTypes.get( a ) );
		assertEquals( 0, cTypes.get( b ) );
		assertEquals( 1, cell.getProportionOfTypeInNeighbourhood( a ) );
		assertEquals( 1, c2.getProportionOfTypeInNeighbourhood( a ) );
		Mature mB = addMature( b );
		assertEquals( 0.5, cell.getProportionOfTypeInNeighbourhood( a ) );
		assertEquals( 0.5, c2.getProportionOfTypeInNeighbourhood( a ) );
		assertEquals( 0.5, cell.getProportionOfTypeInNeighbourhood( b ) );
		assertEquals( 0.5, c2.getProportionOfTypeInNeighbourhood( b ) );
		addMature( a );
		assertEquals( 2.0/3, cell.getProportionOfTypeInNeighbourhood( a ) );
		assertEquals( 1.0/3, cell.getProportionOfTypeInNeighbourhood( b ) );
	}
	
	Mature addMature( AgentType type )
	{
		return addMature( type, cell );
	}
	
	Mature addMature( AgentType type, Cell cell )
	{
		Mature m = new Mature( model, type );
		assertTrue( m.moveTo( cell ) );
		return m;
	}
	
	void createBasic( int size )
	{
		createChangelessWorld( size, true );
		Cell.INITIAL_CAPACITY = 10;
		city.reinitialiseCellCapacities();
		cell = grid.getObjectAt( 0,0 );
	}
	
	

}
