package gi.agents;

import gi.CombinedModel;
import gi.city.*;
import static gi.agents.AgentType.*;

import java.util.*;

import repast.simphony.context.Context;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.Grid;
import repast.simphony.util.ContextUtils;

public class Agent
{
	public static final String SOCIAL_NETWORK = "Social Network";
	public static int MATURATION_AGE = 20;
	public static int OLD_AGE = 50;
	public static int MAX_CELLS = 20;
	int age = 1;
	Cell location = null;
	Context context;
	Demographics demographics;
	CombinedModel model;
	City city;
	Comparator<Cell> comparator;
	AgentType type;
	public static double SATISFACTION_THRESHOLD = 0.0;
	public static double TYPE_PREFERENCE_WEIGHTING = 1.1;
	public static double DENSITY_PREFERENCE_WEIGHTING = 1.0;
	public static double FEATURE_PREFERENCE_WEIGHTING = 1.0;

	public Agent( CombinedModel model )
	{
		this( model, AgentType.getRandomType() );
	}

	public Agent( CombinedModel model, AgentType type )
	{
		super();
		this.model = model;
		this.context = model.getContext();
		this.demographics = model.getDemographics();
		this.city = model.getCity();
		this.type = type;
	}

	/**
	 * Add code in here to check every turn whether the current cell is
	 * satisfactory, using the satisfaciton threshold
	 */
	@ScheduledMethod(start = 1, interval = 1)
	public void checkEvaluation()
	{
		if( location != null && evaluateCell( location ) < SATISFACTION_THRESHOLD )
			tryMoving();
	}

	/**
	 * You should create your evaluation function here, using some weighted combination of
	 * the type- density- and feature preference functions later in this file. If you are using
	 * different types of agents, you can either use if statements (e.g. if( type.equals( A )) ...)
	 * or a case statement, as in the Cell transition rules
	 * @param c
	 * @return
	 */
	public double evaluateCell( Cell c )
	{
		return TYPE_PREFERENCE_WEIGHTING * getTypePreference( c ) +
			DENSITY_PREFERENCE_WEIGHTING * getDensityPreference( c ) + 
			FEATURE_PREFERENCE_WEIGHTING * getFeaturePreference( c );
	}

	/**
	 * This is your feature preference. You can use city.getDistanceTo( <feature>, cell ) to get the
	 * distance to the nearest instance of a feature.
	 * @param c
	 * @return
	 */
	public double getFeaturePreference( Cell c )
	{
		return 0;
	}

	/**
	 * This is a wrapper round the static method - you should use this function
	 * in your evaluation function, but you should put the actual preference function in
	 * the static version below.
	 * @param c
	 * @return
	 */
	public double getTypePreference( Cell c )
	{
		return typeBasedPreference( c, type );
	}

	/**
	 * Create a function here which indicates a preference for neighbourhoods of the
	 * given type.
	 * @param c
	 * @param type
	 * @return
	 */
	public static double typeBasedPreference( Cell c, AgentType type )
	{
		return 0.0;
	}

	/**
	 * This is already overridden in Mature and Old, so don't change it here!
	 * @param c
	 * @return
	 */
	public double getDensityPreference( Cell c )
	{
		return 0.5;
	}


	/*
	 * Useful functions here
	 */

	public void die()
	{
		Context c = getContext();
		remove();
	}
	public void moveOrEmigrate()
	{
		Cell c = findNewLocation();
		//if( c != null && ( location == null || evaluateCell( c ) > evaluateCell( location ) ) && moveTo( c ) )
		if( c != null && moveTo( c ) )
			return;
		//Try to move, otherwise, emigrate
		System.out.println( "No free better cells! Emigrating" );
		model.getCity().unsatisfiedDemmand();
		emigrate();
	}

	public void tryMoving()
	{
		//Try to find a new place to live
		Cell c = findNewLocation();
		if( c == null )
			return;
		if( evaluateCell( c ) > evaluateCell( location ) )
			moveTo( c );
	}

	/*
	 * You probably don't need functions under here...
	 */

	public void setAge( int age )
	{
		this.age = age;
	}

	public int getAge()
	{
		return this.age;
	}

	Context getContext()
	{
		Context c = (Context) ContextUtils.getContext( this );
		return c;
	}

	void remove()
	{
		if( location != null )
			location.remove( this );
		//else System.out.println( "Couldn't remove agent from location! (" + location +")" );
		location = null;

		Context<Agent> c = getContext();
		if( c != null )
			c.remove( this );
		else
			System.out.println( "Couldn't remove agent from context!" );
		if( c != null && c.contains( this ) )
			System.out.println( "Still in context after removal!" );

	}

	public void emigrate()
	{
		System.out.println( "Agent emigrating" );
		remove();
	}

	public boolean moveTo( Cell c )
	{
		if( c.add( this ) )
		{
			if( location != null )
				location.remove( this );
			location = c;
			return true;
		}
		return false;
	}

	public Cell getLocation()
	{
		return location;
	}

	Cell findNewLocation()
	{
		List<Cell> cells = new ArrayList( MAX_CELLS );
		for ( Cell c : city.getRandomAvailableCells( MAX_CELLS ) )
			cells.add( c );
		Collections.sort( cells, getComparator() );
		Collections.reverse( cells );
		for ( Cell c : cells )
		{
			if( c.hasFreeCapacity() )
				return c;
		}
		return null;
	}

	public Comparator<Cell> getComparator()
	{
		if( comparator == null )
			comparator = new Comparator<Cell>()
			{
				public int compare( Cell o1, Cell o2 )
				{
					return Double.compare( evaluateCell( o1 ), evaluateCell( o2 ) );
				}

			};

		return comparator;
	}

	public void setType( AgentType type )
	{
		this.type = type;
	}

	public AgentType getType()
	{
		return type;
	}

}
