package gi.city;

import static java.lang.Math.min;
import gi.CombinedModel;
import static gi.city.CellState.*;

import java.util.*;

import repast.simphony.context.Context;
import repast.simphony.context.space.grid.*;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.*;
import repast.simphony.parameter.Parameters;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.grid.*;

public class City
{
	Grid<Cell> grid;
	Context context;
	List<int[]> CENTRAL_POINT = new ArrayList<int[]>();
	List<int[]> LINE = new ArrayList<int[]>();
	List<int[]> CIRCLE = new ArrayList<int[]>();
	List<int[]> STAR = new ArrayList<int[]>();
	List<Cell> cells = new ArrayList<Cell>();
	int size;
	CombinedModel model;
	int totalCapacity = 0;
	int totalFreeCapacity = 0;
	int totalPopulation;
	int newHousesThisTick = 0;
	int unsatisfiedDemand = 0;
	int unsatisfiedDemandCounter = 0;
	public static boolean CREATE_RIVER = true;
	public static int RIVER_OFFSET = 4;
	Set<CityFeature> features = new HashSet<CityFeature>();
	public static final double floodplain100Years = 2;
	public static final double floodplain50Years = 1;
	
	public static final String CITY_GRID = "CityGrid";
	
	public City( CombinedModel model )
	{
		this.model = model;
	}
	
	public void build( Context context )
	{
		this.context = context;
		init();
		for( int[] coords : CENTRAL_POINT ) setState( NEW, coords );
		context.add( this );
		//RunState.getInstance().getScheduleRegistry().getModelSchedule().schedule( this );
	}
	
	/**
	 * Basic initialisation - put your feature addition code here! You can use
	 * addFeature( name, x, y ) to add a feature. Each cell can only have one feature in
	 * it. See examples below.
	 */
	public void init()
	{
		//get the size of the grid from the environment
		Parameters p = RunEnvironment.getInstance().getParameters();
		size = (Integer) p.getValue( "worldSize" );
		// Create a new 2D grid 
		GridFactory factory =  GridFactoryFinder.createGridFactory( null );
		grid = factory.createGrid( "CityGrid", context, new GridBuilderParameters<Cell>( new StrictBorders(), new SimpleGridAdder<Cell>(), false, size, size ) );

		for( int i = 0; i < size; i++ )
			for( int j = 0; j < size; j++ )
			{
				Cell c = new Cell( model );
				context.add( c );
				grid.getAdder().add( grid, c );
				grid.moveTo( c, i, j );
				c.setLocation( grid.getLocation( c ));
				cells.add( c );
			}
		
		initialiseShapes();
		if( CREATE_RIVER ) createRiver();
		//addFeature( "Spaceport", 5, 5 );
		//addFeature( "Stripmine", 15, 5 );
	}
	
	private void createRiver()
	{
		Set<Cell> riverCells = new HashSet<Cell>( size );
		for( int i = 0; i < size; i++ )
		{
			riverCells.add( setState( RIVER, RIVER_OFFSET, i ) );
		}
		for( Cell c : cells )
		{
			if( riverCells.contains( c ))
			{
				c.setDistanceToRiver( 0 );
				continue;
			}
			double dist = Double.MAX_VALUE;
			for( Cell r : riverCells )
			{
				double d = grid.getDistance( c.getLocation(), r.getLocation() );
				if( d < dist ) dist = d;
			}
			c.setDistanceToRiver( dist );
		}
	}

	@ScheduledMethod( start = 1.1, interval=1, priority=0 )
	public void update()
	{
		System.out.printf( "NewBuilds: %d, Demand: %d\n", newHousesThisTick, unsatisfiedDemand );
		newHousesThisTick = 0;
		//Update global numbers etc. here
		unsatisfiedDemand = unsatisfiedDemandCounter;
		unsatisfiedDemandCounter = 0;
	}
	
	@ScheduledMethod( start = 1, interval=1, priority=ScheduleParameters.LAST_PRIORITY )
	public void printState()
	{
		//printCity();
		totalCapacity = 0;
		totalFreeCapacity = 0;
		totalPopulation = 0;
		for( Cell c : cells )
		{
			if( c.isHabitable() ) totalCapacity += c.getCapacity();
			totalFreeCapacity += c.getFreeCapacity();
			totalPopulation += c.getPopulation();
		}
		
	}
	
	/**
	 * Sets the state of one of the cells in the grid
	 * @param state
	 * @param pos
	 */
	Cell setState(  CellState state, int... pos )
	{
		Cell c = grid.getObjectAt( pos );
		c.setState( state );
		c.updateState();
		return c;
	}
	
	/**
	 * Initialises the possible starting shapes
	 */
	void initialiseShapes()
	{
		int minDim = min( size, size );
		int cx = size/2;
		int cy = size/2;
		
		//Central point is just one coordinate
		CENTRAL_POINT.add( new int[]{ cx, cy } );
		//Horizontal line from 1/4 to 3/4 of the width
		for( int i = size/4; i < 3*size/4; i++ )
			LINE.add( new int[]{i, cy } );
		
		//All points within 1/6 of the centre
		for( int i = 0; i < size; i++ )
			for( int j = 0; j < size; j++ )
				if( ( cx - i ) * (cx-i) + (cy-j)* (cy-j) < minDim / 6 )
					CIRCLE.add( new int[] { i, j } );
		
		//Points on diagonals
		STAR.add( new int[]{ cx, cy } );
		for( int i = 1; i < minDim/6; i++ )
		{
			STAR.add( new int[]{ cx-i, cy-i } );
			STAR.add( new int[]{ cx+i, cy-i } );
			STAR.add( new int[]{ cx+i, cy+i } );
			STAR.add( new int[]{ cx-i, cy+i } );
		}
		
	}
	
	public Iterable<Cell> getCells()
	{
		return cells;
	}
	
	public Iterable<Cell> getRandomCells( int number )
	{
		return context.getRandomObjects( Cell.class, number );
	}
	
	public Iterable<Cell> getRandomAvailableCells( int max )
	{
		List<Cell> available = new ArrayList<Cell>(cells.size());
		for( Cell c : cells ) 
			if( c.hasFreeCapacity() ) available.add( c );
		while( available.size() > max )
			available.remove( RandomHelper.nextIntFromTo( 0, available.size()-1 ) );
		Collections.shuffle( available );
		return available;
	}
	
	void printCity()
	{
		System.out.println( "Printing city: " + this );
		for( int i = size-1; i >= 0; i-- )
		{
			System.out.println();
			for( int j = 0; j < size; j++ )
			{
				Cell c = grid.getObjectAt( j,i );
				System.out.printf( c.getSummary() );
			}
		}
			System.out.println();
	}
	
	public void reinitialiseCellCapacities()
	{
		for( Cell c : getCells() ) c.capacity = Cell.INITIAL_CAPACITY;
	}

	public int getTotalCapacity()
	{
		return totalCapacity;
	}

	public int getTotalFreeCapacity()
	{
		return totalFreeCapacity;
	}

	public int getTotalPopulation()
	{
		return totalPopulation;
	}

	public int getNewHousesThisTick()
	{
		return newHousesThisTick;
	}
	
	public void addHouses( int number )
	{
		newHousesThisTick += number;
	}
	
	public void unsatisfiedDemmand()
	{
		unsatisfiedDemandCounter++;
	}

	public int getUnsatisfiedDemand()
	{
		return unsatisfiedDemand;
	}

	public Grid<Cell> getGrid()
	{
		return grid;
	}

	public int getSize()
	{
		return size;
	}
	
	class CityFeature
	{
		String name;
		Cell cell;
		
		public CityFeature( String name, int... location )
		{
			this.name = name;
			cell = grid.getObjectAt( location );
			cell.setFeature( name );
		}
	}
	
	public double getDistanceTo( String featureName, int... pos )
	{
		return getDistanceTo( featureName, grid.getObjectAt( pos ) );
	}
	public double getDistanceTo( String featureName, Cell c )
	{
		double distance = Double.POSITIVE_INFINITY;
		for( CityFeature f : features )
		{
			if( ! f.name.equals( featureName )) continue;
			double d = grid.getDistance( c.getLocation(), f.cell.getLocation() );
			if( d < distance ) distance = d;
		}
		return distance;
	}
	
	public void addFeature( String featurename, int... position )
	{
		Cell c = grid.getObjectAt( position );
		Iterator<CityFeature> it = features.iterator();
		while( it.hasNext() )
		{
			CityFeature f = it.next();
			if( f.cell == c ) it.remove();
		}
		features.add( new CityFeature( featurename, position ) );
	}
	
	public String getFeature( int... position )
	{
		return grid.getObjectAt( position ).getFeature();
	}
	
	public int getNumFeatures() { return features.size(); }
	
	public int getPopulationIn100YearFloodplain()
	{
		return getPopulationNearRiver( floodplain100Years );
	}
	public int getPopulationIn50YearFloodplain()
	{
		return getPopulationNearRiver( floodplain50Years );
	}
	
	public int getPopulationNearRiver( double distance )
	{
		int pop = 0;
		for( Cell c : cells )
			if( c.getDistanceToRiver() <= distance )
				pop += c.getPopulation();
		return pop;
	}
}
