package gi.city.display;

import gi.city.Cell;

import java.awt.*;

/**
 * Displays the state of the cells, with:
 * NEW -> red
 * AVAILABLE -> yellow
 * EMPTY -> gray
 * OLD -> orange fading to green as age approaches redevelopment age
 * RIVER -> blue
 * 
 * Bordered cells are in the floodplain, red borders in the 100year, orange borders = 50 year.
 * @author dave
 *
 */
public class CellStateStyle extends AbstractCellStyle
{
	Stroke s1 = new BasicStroke( 1 );
	Color f50 = new Color( 1f, 0f, 0f, 0.4f );
	Color f100 = new Color( 1f, 0.5f, 0f, 0.4f );

	@Override
	Color getColor( Cell agent )
	{
		float ageProp = (float)agent.getAge()/Cell.REDEVELOPMENT_AGE;
		if( ageProp > 1 ) ageProp = 1;
		if( ageProp < 0 ) ageProp = 0;
		switch ( agent.getState() )
		{
		case NEW:
			return Color.red;
		case AVAILABLE:
			return Color.WHITE;
		case OLD:
			return new Color( 1-ageProp, 0.5f+ageProp/2, 0 );
		case EMPTY:
			return Color.gray;
		case RIVER:
			return Color.blue;
		default:
			return Color.black;
		}
	}
	
	public Stroke getStroke(Cell agent) {
		
		if( agent.getDistanceToRiver() <= 2 && agent.getDistanceToRiver() > 0.01 ) return s1;
		return stroke;
	}
	public Paint getStrokePaint(Cell agent) {
		if( agent.getDistanceToRiver() < 1.01 ) return f50;
		if( agent.getDistanceToRiver() < 2.01 ) return f100;
		return strokePaint;
	}

}
