package gi;

import gi.city.City;

import org.junit.*;

import static java.lang.Math.sqrt;
import static org.junit.Assert.*;

public class FeatureDistanceTest extends ModelTest
{
	String f1 = "Feature1";
	String f2 = "Feature2";
	
	@Test
	public void testSingleFeatureDistances()
	{
		createChangelessWorld( 20 );
		city.addFeature( f1, 5,5 );
		
		assertEquals( sqrt( 5*5*2  ), city.getDistanceTo( f1, 0, 0 ), 0.01 );
		assertEquals( sqrt( 0  ), city.getDistanceTo( f1, 5, 5 ), 0.01 );
		assertEquals( Double.POSITIVE_INFINITY, city.getDistanceTo( f2, 0, 0 ), 0.01);
	}
	
	@Test
	public void testFeatureReplacement()
	{
		createChangelessWorld( 20 );
		city.addFeature( f1, 5,5 );
		
		assertEquals( sqrt( 5*5*2  ), city.getDistanceTo( f1, 0, 0 ), 0.01 );
		assertEquals( sqrt( 0  ), city.getDistanceTo( f1, 5, 5 ), 0.01 );
		
		city.addFeature( f1, 5, 5 );
		assertEquals( 1, city.getNumFeatures() );
		assertEquals( sqrt( 5*5*2  ), city.getDistanceTo( f1, 0, 0 ), 0.01 );
		assertEquals( sqrt( 0  ), city.getDistanceTo( f1, 5, 5 ), 0.01 );
		
		city.addFeature( f2, 5, 5 );
		assertEquals( sqrt( 5*5*2  ), city.getDistanceTo( f2, 0, 0 ), 0.01 );
		assertEquals( sqrt( 0  ), city.getDistanceTo( f2, 5, 5 ), 0.01 );
		assertEquals( Double.POSITIVE_INFINITY, city.getDistanceTo( f1, 0, 0 ), 0.01);
	}
	
	@Test
	public void testDoubleFeatures()
	{
		createChangelessWorld( 20 );
		city.addFeature( f1, 5,5 );
		
		assertEquals( sqrt( 5*5*2  ), city.getDistanceTo( f1, 0, 0 ), 0.01 );
		assertEquals( 5, city.getDistanceTo( f1, 0, 5 ), 0.01 );
		assertEquals( 5, city.getDistanceTo( f1, 5, 0 ), 0.01 );
		assertEquals( sqrt( 0  ), city.getDistanceTo( f1, 5, 5 ), 0.01 );
		assertEquals( sqrt( 5*5*2  ), city.getDistanceTo( f1, 10, 10 ), 0.01 );
		
		
		city.addFeature( f1, 2,2 );
		assertEquals( sqrt( 2*2*2  ), city.getDistanceTo( f1, 0, 0 ), 0.01 );
		assertEquals( sqrt( 0  ), city.getDistanceTo( f1, 5, 5 ), 0.01 );
		assertEquals( sqrt( 5*5*2  ), city.getDistanceTo( f1, 10, 10 ), 0.01 );
		
	}
	

}
